@@ -5,14 +5,14 @@
 //! For example:
 //!
 //! ```rust
-//! use mmarinus::{Kind, Map, perms};
+//! use mmarinus::{Map, perms};
 //!
 //! let mut zero = std::fs::File::open("/dev/zero").unwrap();
 //!
 //! let map = Map::map(32)
 //!     .near(128 * 1024 * 1024)
 //!     .from(&mut zero, 0)
-//!     .known::<perms::Read>(Kind::Private)
+//!     .map(perms::Read)
 //!     .unwrap();
 //!
 //! assert_eq!(&*map, &[0; 32]);
@@ -21,21 +21,21 @@
 //! You can also remap an existing mapping:
 //!
 //! ```rust
-//! use mmarinus::{Kind, Map, perms};
+//! use mmarinus::{Map, perms};
 //!
 //! let mut zero = std::fs::File::open("/dev/zero").unwrap();
 //!
 //! let mut map = Map::map(32)
 //!     .anywhere()
 //!     .from(&mut zero, 0)
-//!     .known::<perms::Read>(Kind::Private)
+//!     .map(perms::Read)
 //!     .unwrap();
 //!
 //! assert_eq!(&*map, &[0; 32]);
 //!
 //! let mut map = map.remap()
 //!     .from(&mut zero, 0)
-//!     .known::<perms::ReadWrite>(Kind::Private)
+//!     .map(perms::ReadWrite)
 //!     .unwrap();
 //!
 //! assert_eq!(&*map, &[0; 32]);
@@ -48,19 +48,19 @@
 //! Alternatively, you can just change the permissions:
 //!
 //! ```rust
-//! use mmarinus::{Kind, Map, perms};
+//! use mmarinus::{Map, perms};
 //!
 //! let mut zero = std::fs::File::open("/dev/zero").unwrap();
 //!
 //! let mut map = Map::map(32)
 //!     .at(128 * 1024 * 1024)
 //!     .from(&mut zero, 0)
-//!     .known::<perms::Read>(Kind::Private)
+//!     .map(perms::Read)
 //!     .unwrap();
 //!
 //! assert_eq!(&*map, &[0; 32]);
 //!
-//! let mut map = map.reprotect::<perms::ReadWrite>().unwrap();
+//! let mut map = map.reprotect(perms::ReadWrite).unwrap();
 //!
 //! assert_eq!(&*map, &[0; 32]);
 //! for i in map.iter_mut() {
@@ -72,9 +72,9 @@
 //! Mapping a whole file into memory is easy:
 //!
 //! ```rust
-//! use mmarinus::{Kind, perms};
+//! use mmarinus::{Map, perms};
 //!
-//! let map = Kind::Private.load::<perms::Read, _>("/etc/os-release").unwrap();
+//! let map = Map::load("/etc/os-release", perms::Read).unwrap();
 //! ```
 
 #![deny(clippy::all)]
@@ -91,12 +91,20 @@ use std::slice::{from_raw_parts, from_raw_parts_mut};
 mod sealed {
     pub trait Stage {}
 
-    pub trait Type {}
+    pub trait Type {
+        fn perms(self) -> libc::c_int;
+    }
 
     pub trait Known: Type {
         const VALUE: libc::c_int;
     }
 
+    impl<T: Known> Type for T {
+        fn perms(self) -> libc::c_int {
+            Self::VALUE
+        }
+    }
+
     pub trait Readable: Known {}
 
     pub trait Writeable: Known {}
@@ -114,9 +122,7 @@ pub mod perms {
         ($($name:ident[$($trait:ident),* $(,)?] => $value:expr),+ $(,)?) => {
             $(
                 #[derive(Debug)]
-                pub struct $name(());
-
-                impl super::Type for $name {}
+                pub struct $name;
 
                 impl super::Known for $name {
                     const VALUE: libc::c_int = $value;
@@ -140,8 +146,14 @@ pub mod perms {
         ReadWriteExecute[Readable, Writeable, Executable] => libc::PROT_READ | libc::PROT_WRITE | libc::PROT_EXEC,
     }
 
-    pub struct Unknown(());
-    impl super::Type for Unknown {}
+    pub struct Unknown(pub libc::c_int);
+
+    impl super::Type for Unknown {
+        #[inline]
+        fn perms(self) -> libc::c_int {
+            self.0
+        }
+    }
 }
 
 enum Address {
@@ -203,33 +215,6 @@ impl From<ErrorKind> for Error<()> {
     }
 }
 
-/// The type of mapping to create
-#[derive(Copy, Clone, Debug)]
-pub enum Kind {
-    /// A private mapping
-    ///
-    /// See `MAP_PRIVATE` in `man mmap` for more details.
-    Private,
-
-    /// A shared mapping
-    ///
-    /// See `MAP_SHARED` in `man mmap` for more details.
-    Shared,
-}
-
-impl Kind {
-    /// Maps a whole file into memory
-    ///
-    /// This is simply a convenience function.
-    #[inline]
-    pub fn load<T: Known, U: AsRef<Path>>(self, path: U) -> Result<Map<T>, Error<()>> {
-        let err = Err(ErrorKind::InvalidData);
-        let mut file = std::fs::File::open(path)?;
-        let size = file.metadata()?.len().try_into().or(err)?;
-        Map::map(size).anywhere().from(&mut file, 0).known(self)
-    }
-}
-
 #[doc(hidden)]
 pub struct Size<T> {
     prev: T,
@@ -352,22 +337,15 @@ impl<'a, T> Builder<Source<'a, T>> {
         self
     }
 
-    /// Creates a mapping with unknown (i.e. run-time) permissions
+    /// Creates a mapping with the specified permissions
     ///
-    /// You should use a moderate amount of effort to avoid using this method.
-    /// Its purpose is to allow dynamically setting the map permissions at run
-    /// time. This implies that we cannot do compile-time checking.
-    ///
-    /// The resulting `Map` object will be missing a lot of useful APIs.
-    /// However, it will still manage the map lifecycle.
+    /// The use of `Known` permissions should be preferred to the use of
+    /// `Unknown` (i.e. runtime) permissions as this will supply a variety of
+    /// useful APIs.
     #[inline]
-    pub fn unknown(self, kind: Kind, perms: i32) -> Result<Map<perms::Unknown>, Error<T>> {
+    pub fn map<U: Type>(self, perms: U) -> Result<Map<U>, Error<T>> {
         let einval = ErrorKind::InvalidInput.into();
-
-        let kind = match kind {
-            Kind::Private => libc::MAP_PRIVATE,
-            Kind::Shared => libc::MAP_SHARED,
-        };
+        let perms = perms.perms();
 
         let huge = match self.0.huge {
             Some(x) if x & !libc::MAP_HUGE_MASK != 0 => {
@@ -400,7 +378,7 @@ impl<'a, T> Builder<Source<'a, T>> {
         };
 
         let size = self.0.prev.prev.size;
-        let flags = kind | fixed | anon | huge;
+        let flags = libc::MAP_PRIVATE | fixed | anon | huge;
 
         let ret = unsafe { libc::mmap(addr as _, size, perms, flags, self.0.fd, self.0.offset) };
         if ret == libc::MAP_FAILED {
@@ -418,22 +396,6 @@ impl<'a, T> Builder<Source<'a, T>> {
             data: PhantomData,
         })
     }
-
-    /// Creates a mapping with known (i.e. compile-time) permissions
-    ///
-    /// The use of this method should be preferred to `Self::unknown()` since
-    /// this permits for compile-time permission validation.
-    #[inline]
-    pub fn known<U: Known>(self, kind: Kind) -> Result<Map<U>, Error<T>> {
-        let unknown = self.unknown(kind, U::VALUE)?;
-        let known = Map {
-            addr: unknown.addr,
-            size: unknown.size,
-            data: PhantomData,
-        };
-        forget(unknown);
-        Ok(known)
-    }
 }
 
 /// A smart pointer to a mapped region of memory
@@ -500,6 +462,17 @@ impl<T: Known> From<Map<T>> for Map<perms::Unknown> {
 }
 
 impl<T: Type> Map<T> {
+    /// Maps a whole file into memory
+    ///
+    /// This is simply a convenience function.
+    #[inline]
+    pub fn load<U: AsRef<Path>>(path: U, perms: T) -> Result<Map<T>, Error<()>> {
+        let err = Err(ErrorKind::InvalidData);
+        let mut file = std::fs::File::open(path)?;
+        let size = file.metadata()?.len().try_into().or(err)?;
+        Map::map(size).anywhere().from(&mut file, 0).map(perms)
+    }
+
     /// Gets the address of the mapping
     #[inline]
     pub fn addr(&self) -> usize {
@@ -532,8 +505,8 @@ impl<T: Type> Map<T> {
     /// Upon success, the new mapping "steals" the mapping from the old `Map`
     /// instance. Using the old instance is a logic error, but is safe.
     #[inline]
-    pub fn reprotect<U: Known>(self) -> Result<Map<U>, Error<Self>> {
-        if unsafe { libc::mprotect(self.addr as _, self.size, U::VALUE) } != 0 {
+    pub fn reprotect<U: Type>(self, perms: U) -> Result<Map<U>, Error<Self>> {
+        if unsafe { libc::mprotect(self.addr as _, self.size, perms.perms()) } != 0 {
             return Err(Error {
                 map: self,
                 err: std::io::Error::last_os_error(),
@@ -556,14 +529,14 @@ impl<T: Type> Map<T> {
     ///
     /// # Example
     /// ```
-    /// use mmarinus::{Kind, Map, perms};
+    /// use mmarinus::{Map, perms};
     ///
     /// const SIZE: usize = 4 * 1024 * 1024;
     ///
     /// let map = Map::map(SIZE * 2)
     ///     .anywhere()
     ///     .anonymously()
-    ///     .known::<perms::Read>(Kind::Private)
+    ///     .map(perms::Read)
     ///     .unwrap();
     ///
     /// let (l, r) = map.split(SIZE).unwrap();
@@ -603,14 +576,14 @@ impl<T: Type> Map<T> {
     ///
     /// # Example
     /// ```
-    /// use mmarinus::{Kind, Map, perms};
+    /// use mmarinus::{Map, perms};
     ///
     /// const SIZE: usize = 4 * 1024 * 1024;
     ///
     /// let map = Map::map(SIZE * 2)
     ///     .anywhere()
     ///     .anonymously()
-    ///     .known::<perms::Read>(Kind::Private)
+    ///     .map(perms::Read)
     ///     .unwrap();
     ///
     /// let at = map.addr() + SIZE;
@@ -639,7 +612,7 @@ impl Map<perms::Unknown> {
 
 #[cfg(test)]
 mod tests {
-    use crate::{perms, Kind, Map};
+    use crate::{perms, Map};
 
     #[test]
     fn zero_split() {
@@ -648,7 +621,7 @@ mod tests {
         let map = Map::map(SIZE)
             .anywhere()
             .anonymously()
-            .known::<perms::Read>(Kind::Private)
+            .map(perms::Read)
             .unwrap();
 
         let at = map.addr();
@@ -664,7 +637,7 @@ mod tests {
         let map = Map::map(SIZE)
             .anywhere()
             .anonymously()
-            .known::<perms::Read>(Kind::Private)
+            .map(perms::Read)
             .unwrap();
 
         let at = map.addr() + SIZE;
